#!/usr/local/bin/perl

use warnings;
use strict;

my $svgheader = qq(<?xml version="1.0" standalone="no"?>
<!DOCTYPE svg PUBLIC "-//W3C//DTD SVG 1.1//EN" 
  "http://www.w3.org/Graphics/SVG/1.1/DTD/svg11.dtd">
<svg 
   xmlns="http://www.w3.org/2000/svg" version="1.1"
   xmlns:inkscape="http://www.inkscape.org/namespaces/inkscape">
);
my $startline = "<path fill=\"none\" stroke=\"%s\" stroke-width=\"2\"\ \nd=\"";
my $startgroup = qq(<g inkscape:label="%s" inkscape:groupmode="layer" id="%s">\n);
my $endgroup = '</g>';
my $svg;

my $version = "1.8";

my $current_group = 0;
my $total_groups = 0;
my @group_data;
my $current_line = "";
my $slicemode = 0;
my @slices;

my @max_x = (-99999,0);
my @max_y = (0,-99999);
my @min_x = (99999,0);
my @min_y = (0,99999);
my $max_z = -99999;
my $min_z = 99999;
my $avg_z;
my $line_points;
my $page_width = 744;     # As Defined by Inkscap
my $page_height = 1050;   # As Defined by Inkscap
my $current_color = 'black';  # Default Color
my $map_style_name = 'No Map Style Name Found';


if (@ARGV < 2) { usage(); }

my $logfile = shift @ARGV;
my $outfile = shift @ARGV;
 
open(LOGFILE, "<$logfile") || die "ERROR: Can't open input file '$logfile'\n";
open(OUTFILE, ">$outfile") || die "ERROR: Can't open output file '$outfile'\n";

if(@ARGV > 2)  {
	@slices = sort grep(/^[-+]?([0-9]*\.)?[0-9]+$/,@ARGV);  #Grab numbers and only numbers
	$slicemode = 1;
}

while (<LOGFILE>) {
	if (m/Map style name: (.+)/) {
		$map_style_name = $1;
	}
    if (m/ start new map line/) {
    	if($slicemode && $line_points)  {
    		$avg_z /= $line_points;
    		my $i = 0;
    		foreach my $slice (@slices) { last unless($slice <= $avg_z); $i++;}
    		$current_group = $i;
    		$avg_z = 0;
    		$line_points = 0;
    		#print OUTFILE "Current Group: $i\n";
    	}
       	$group_data[$current_group] .= "::$current_color&$current_line";
		$current_line = "";
    } elsif (m/^ mapgroup/ && !$slicemode)  {
    	$current_group = ++$total_groups;
    } elsif (m/ color (\w+)/)  {
    	$current_color = $1;
    } elsif (m/ color$/)  {
    	$current_color = 'black';    
    } elsif (m/Your location is (.*?\.\d{2}), (.*?\.\d{2}), (.*?\.\d{2}).  Your/) {

        my $x = $1;
        my $y = $3;
        my $z = $2;

        $x =~ s/,//g;
        $y =~ s/,//g;
        $z =~ s/,//g;
        
        if ($x < 0) {
            $x = abs($x);  #  poor mans number inversion (changing neg numbers to pos, and vice versa)
        } elsif ($x > 0) {
            $x = (0 - abs($x));
        }
        
        # Get max/min values
        @max_x = ($x,$y) if($x > $max_x[0]);
        @max_y = ($x,$y) if($y > $max_y[1]);
        @min_x = ($x,$y) if($x < $min_x[0]);
        @min_y = ($x,$y) if($y < $min_y[1]);
        
        $min_z = $z if($z < $min_z);
        $max_z = $z if($z > $max_z);
        
        $avg_z += $z;
        $line_points++;

        $current_line .= "$x,$y,";
    }
}

if($slicemode  && $line_points)  {
	$avg_z /= $line_points;
	my $i = 0;
	foreach my $slice (@slices) { last unless($slice <= $avg_z); $i++;}
	$current_group = $i;
	$avg_z = 0;
	$line_points = 0;
}
$group_data[$current_group] .= "::$current_color&$current_line";

my $extents_lines;

$extents_lines  = "::blue&$min_x[0]," . ($min_y[1] - abs($max_x[0] - $min_x[0])*.025). ",$min_x[0]," . ($min_y[1] + abs($max_x[0] - $min_x[0])*.025). ",";
$extents_lines .= "::blue&" . ($min_x[0] - abs($max_x[0] - $min_x[0])*.025) . ",$min_y[1]," . ($min_x[0] + abs($max_x[0] - $min_x[0])*.025) . ",$min_y[1],";
$extents_lines .= "::blue&$max_x[0]," . ($max_y[1] + abs($max_x[0] - $min_x[0])*.025) . ",$max_x[0]," . ($max_y[1] - abs($max_x[0] - $min_x[0])*.025) . ",";
$extents_lines .= "::blue&" . ($max_x[0] + abs($max_x[0] - $min_x[0])*.025). ",$max_y[1]," . ($max_x[0] - abs($max_x[0] - $min_x[0])*.025). ",$max_y[1],";

unshift (@group_data, $extents_lines);

my $x_scale = $page_width/abs($max_x[0] - $min_x[0]);
my $y_scale = $page_height/abs($max_y[1] - $min_y[1]);
my $scale_factor = ($x_scale <= $y_scale) ? $x_scale : $y_scale;
$scale_factor *= .95;  # Scale it so it doesn't hit the edges.

my $x_center = $max_x[0] - (abs($max_x[0] - $min_x[0])/2);
my $y_center = $max_y[1] - (abs($max_y[1] - $min_y[1])/2);

my $prefix;
my @maplines;
my $group_num = 0;
my $group;
my $line;
my $line_count;

print OUTFILE $svgheader;

GROUP:
foreach $group (@group_data) {
	next GROUP unless($group);  # Skip empty groups
	@maplines = split('::', $group);
	print OUTFILE sprintf($startgroup,"group_$group_num", "layer$group_num"); # Start the group
    $group_num++;
    
    LINE:
	foreach $line (@maplines) {
		if($line)  {
			my $color; my $line_data;

			($color, $line_data) = split('&',$line);  # Break out color data
			
			next LINE unless ($line_data);  # Skip blank lines
			my @cords; my $count; my $x; my $y;
			$prefix = 'M';
			print OUTFILE sprintf($startline, $color);
		    		    
		    # Break each line into its cordinates, scale them, and export them.
			@cords = split(',', $line_data);
			for($count = 0; $count < @cords; $count +=2) {
				$x = ($cords[$count] - $x_center) * $scale_factor + ($page_width/2);
				$y = ($cords[$count+1] - $y_center) * $scale_factor + ($page_height/2);
			    print OUTFILE sprintf("%s%.2f,%.2f",$prefix,$x,$y);
			    $prefix = 'L';
			}
			$line_count++;
		    print OUTFILE "\" \nid=\"NewLine$line_count\"/>\n\n";
		}
	}
	print OUTFILE "</g>\n\n"; # Close the group
}

print OUTFILE sprintf($startgroup,"layer$group_num", "text_group"); # Start the group
print OUTFILE qq(  <text x="10" y="20" font-family="Verdana" font-size="18" fill="blue" >
    Mapper2 v$version - $logfile
  </text>
  <text x="10" y="40" font-family="Verdana" font-size="18" fill="blue" >
    Map Stlye Name: $map_style_name
  </text>
  <text x="10" y="$page_height" font-family="Verdana" font-size="18" fill="black" >);
print OUTFILE "<tspan fill=\"blue\">UL: </tspan>" . -$min_x[0] . ", $min_y[1]  \n" .
              "<tspan fill=\"blue\">LR: </tspan>" . -$max_x[0] . ",$max_y[1] \n" . 
              "<tspan fill=\"blue\" dx=\"20\">Max Ele:</tspan> $max_z \n" .
              "<tspan fill=\"blue\">Min Ele:</tspan> $min_z \n" .
  	          "</text>";
print OUTFILE "</g>\n\n"; # Close the group

print OUTFILE "</svg>";

sub usage {
print qq(Mapper v$version by Rex 'SaintPeter' Schrader
	Parses EQ2 .log file, extracting /loc coordinates and converting them
	into a SVG (Scalable Vector Graphics) file.
	
	Commands:
		Mapper2 takes commands as /em emotes from the logfile.
		/em start new map line
			Ends the current line (picks up the pen)
		/em color <svg color name>
			Changes the color of the current and subsequent lines to 
			<svg color name>, e.g. 'black', 'blue', 'green', etc.
		/em color
			Resets the line draw color back to black.
		/em mapgroup
			Creates a new group of lines.  Each group will be on it`s own layer
			in the svg file.  Use to seperate floors or areas when mapping.
			
	EQ2 Setup:
		/log on
			Turns on Logging
		show_map_style_name 1
			If this command is enabled, mapper will automatically extract the 
			map style name when you zone in.

	Usage:
		SVG Generation Mode
			mapper <inputfile.txt> <outputfile.svg>
		Splitter Mode
			mapper <inputfile.txt> <outputfile.svg> [elev 1] [elev 2] [elev n]
);
exit(0);
}